/***********************************************************************
*
* Copyright (c) 2021-2022 Barbara Geller and Ansel Sermersheim
*
* This example is free software, released under the BSD 2-Clause license.
* For license details refer to LICENSE provided with this project.
*
* CopperSpice is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
* https://opensource.org/licenses/BSD-2-Clause
*
***********************************************************************/

#include <QtCore>
#include <QtGui>

class CustomWidget;

class MainWindow : public QWidget
{
 public:
   MainWindow();

 private:
   CustomWidget *drawRegion;
};

class CustomWidget : public QWidget
{
   CS_OBJECT(CustomWidget)

 public:
   CustomWidget(QWidget *parent = nullptr);

 protected:
   void paintEvent(QPaintEvent *event) override;
};

MainWindow::MainWindow()
{
   setMinimumSize(800, 600);

   drawRegion = new CustomWidget;

   QPushButton *close_pb = new QPushButton();
   close_pb->setText("Close");

   QHBoxLayout *layoutButtons = new QHBoxLayout();
   layoutButtons->addStretch();
   layoutButtons->addWidget(close_pb);
   layoutButtons->addStretch();

   QVBoxLayout *layoutMain = new QVBoxLayout(this);
   layoutMain->setContentsMargins(40, 35, 40, 20);

   layoutMain->addWidget(drawRegion);
   layoutMain->addSpacing(20);
   layoutMain->addLayout(layoutButtons);

   connect(close_pb, &QPushButton::clicked, this, &QWidget::close);
}

CustomWidget::CustomWidget(QWidget *parent)
    : QWidget(parent)
{
   setBackgroundRole(QPalette::Base);
   setAutoFillBackground(true);
}

void CustomWidget::paintEvent(QPaintEvent *)
{
   QPainter painter(this);
   painter.drawText(50, 40, "Text displayed by calling QPainter::drawText()");

   QRect rec1(50, 75, 100, 25);
   painter.drawRect(rec1);
   rec1.adjust(2, 0, 0, 0);
   painter.drawText(rec1, "This text will be truncated to fit in the rectangle");
   painter.drawText(175, 93, "(Full Text) This text will be truncated to fit in the rectangle");

   //
   QRect rec2(50, 145, 550, 40);
   painter.drawRect(rec2);
   rec2.adjust(2, 2, -2, -2);

   QFont font = painter.font();
   font.setPointSize(10);
   painter.setFont(font);

   painter.drawText(rec2, Qt::AlignLeft, "Left Aligned");
   painter.drawText(rec2, Qt::AlignCenter, "Center Aligned");
   painter.drawText(rec2, Qt::AlignRight | Qt::AlignVCenter, "Right VCenter");

   //
   painter.translate(50, 215);

   QRect rec3(0, 0, 550, 200);
   painter.drawRect(rec3);

   QTextDocument doc;
   doc.setTextWidth(rec3.width());

   QString text =
         "<font face='Arial' color='blue' size='5'>"
         "<u>HTML Example</u> </font><br><br>"

         "<font size='4'>"
         "Another way to draw text is by using QTextDocument"
         " and calling setHtml(). <br><br>"
         "Text can be formatted by using standard html tags."
         "</font>";

   doc.setHtml(text);
   doc.drawContents(&painter, rec3);
}

int main(int argc, char *argv[])
{
   QApplication app(argc, argv);

   MainWindow *window = new MainWindow();
   window->show();

   return app.exec();
}
